@@ -119,14 +119,24 @@ static bool sort_clamp_values_dec(zone_trip_limits_t limit_1,
 /*
  * How the state is set?
  * If the state set is called before debounce interval, then simply return
- * success We have two mask zone_mask and trip_mask. The idea is that while
- * any zone and trip has turned on then the other zone or trip can't reset
- * unless it is the last one.
+ * success.
+ * It is possible that same cdev is used by two trips in the same zone or
+ * different zone. So if one trip activated a cdev (state = 1), even if the
+ * other trip calls for deactivation (state = 0), the trip shouldn't be
+ * deactivated. This is implemented by adding each activation to push the
+ * zone, trip and target_value to a list if not present and remove on
+ * call for deactivation. After deleting from the list if the list still
+ * has members, that means that this device is still under activation from
+ * some other trip. So in this case simply return without changing the state
+ * of cdev.
+ *
+ * Special handling when a target value is passed:
+ *
  * In addition a zone trip can call for a particular target value for this
  * cdev (When it doesn't want the exponential or pid control to use, this
  * is true when multiple trips wants to use the cdev with different state
  * values. They way we support this:
- * When a valid target value is set then we push to a list, sorted using
+ * When a valid target value is set then we push to the list, sorted using
  * increasing target values. The passed target value is set, no check is
  * done here to check the state higher/lower than the current state. This
  * is done during trip_point_check class.
@@ -137,119 +147,123 @@ static bool sort_clamp_values_dec(zone_trip_limits_t limit_1,
  */
 
 int cthd_cdev::thd_cdev_set_state(int set_point, int target_temp,
-		int temperature, int state, int zone_id, int trip_id,
-		int target_value) {
+		int temperature, int state, int zone_id, int trip_id, int target_value,
+		bool force) {
 
 	time_t tm;
 	int ret;
 
 	time(&tm);
-	thd_log_debug(">>thd_cdev_set_state index:%d state:%d :%d:%d:%d\n", index,
-			state, zone_id, trip_id, target_value);
-	if (last_state == state && (tm - last_action_time) <= debounce_interval) {
+	thd_log_debug(">>thd_cdev_set_state index:%d state:%d :%d:%d:%d force:%d\n",
+			index, state, zone_id, trip_id, target_value, force);
+	if (!force && last_state == state
+			&& (tm - last_action_time) <= debounce_interval) {
 		thd_log_debug(
 				"Ignore: delay < debounce interval : %d, %d, %d, %d, %d\n",
 				set_point, temperature, index, get_curr_state(), max_state);
 		return THD_SUCCESS;
 	}
+
 	last_state = state;
+
 	if (state) {
-		zone_mask |= (1 << zone_id);
-		trip_mask |= (1 << trip_id);
+		bool found = false;
 
-		if (target_value != TRIP_PT_INVALID_TARGET_STATE) {
-			zone_trip_limits_t limit;
-			bool found = false;
-
-			for (unsigned int i = 0; i < zone_trip_limits.size(); ++i) {
-				if (zone_trip_limits[i].zone == zone_id
-						&& zone_trip_limits[i].trip == trip_id) {
-					found = true;
-					break;
-				}
+		// Search for the zone and trip id in the list
+		for (unsigned int i = 0; i < zone_trip_limits.size(); ++i) {
+			if (zone_trip_limits[i].zone == zone_id
+					&& zone_trip_limits[i].trip == trip_id) {
+				found = true;
+				break;
 			}
-			if (!found) {
-				limit.zone = zone_id;
-				limit.trip = trip_id;
-				limit.target_value = target_value;
-				thd_log_debug("Added zone %d trip %d clamp %d\n", limit.zone,
-						limit.trip, limit.target_value);
-				zone_trip_limits.push_back(limit);
+		}
+		// If not found in the list add to the list
+		if (!found) {
+			zone_trip_limits_t limit;
+
+			limit.zone = zone_id;
+			limit.trip = trip_id;
+			limit.target_value = target_value;
+			thd_log_info("Added zone %d trip %d clamp %d\n", limit.zone,
+					limit.trip, limit.target_value);
+			zone_trip_limits.push_back(limit);
+
+			if (target_value != TRIP_PT_INVALID_TARGET_STATE) {
 				if (min_state < max_state) {
 					std::sort(zone_trip_limits.begin(), zone_trip_limits.end(),
-						sort_clamp_values_asc);
+							sort_clamp_values_asc);
 				} else {
 					std::sort(zone_trip_limits.begin(), zone_trip_limits.end(),
-						sort_clamp_values_dec);
+							sort_clamp_values_dec);
 				}
 			}
-			set_curr_state_raw(target_value, zone_id);
-			curr_state = target_value;
-			last_action_time = tm;
-			thd_log_info(
-					"Set : threshold:%d, temperature:%d, cdev:%d(%s), curr_state:%d, max_state:%d\n",
-					set_point, temperature, index, type_str.c_str(),
-					get_curr_state(), max_state);
-			return THD_SUCCESS;
 		}
 	} else {
-
-		if (zone_mask & (1 << zone_id)) {
-			if (trip_mask & (1 << trip_id)) {
-				trip_mask &= ~(1 << trip_id);
-				zone_mask &= ~(1 << zone_id);
-			}
-		}
-
+		thd_log_debug("zone_trip_limits.size() %lu\n", zone_trip_limits.size());
 		if (zone_trip_limits.size() > 0) {
 			int length = zone_trip_limits.size();
+			int _target_value;
 			int i;
 
 			// Just remove the current zone requesting to turn off
 			for (i = 0; i < length; ++i) {
+				thd_log_info("match zone %d trip %d clamp %d\n",
+						zone_trip_limits[i].zone, zone_trip_limits[i].trip,
+						zone_trip_limits[i].target_value);
+
 				if (zone_trip_limits[i].zone == zone_id
-						&& zone_trip_limits[i].trip == trip_id) {
+						&& zone_trip_limits[i].trip == trip_id
+						&& target_value == zone_trip_limits[i].target_value) {
+					_target_value = zone_trip_limits[i].target_value;
 					zone_trip_limits.erase(zone_trip_limits.begin() + i);
-					thd_log_debug("Erased  [%d: %d\n", zone_id, trip_id);
+					thd_log_info("Erased  [%d: %d %d\n", zone_id, trip_id,
+							target_value);
 					break;
 				}
 			}
-			zone_trip_limits_t limit;
 
-			if (zone_trip_limits.size() == 0) {
-				limit.target_value = get_min_state();
-				limit.zone = zone_id;
-				limit.trip = trip_id;
-			} else {
+			if (zone_trip_limits.size()) {
+				zone_trip_limits_t limit;
+
 				limit = zone_trip_limits[zone_trip_limits.size() - 1];
-			}
+				target_value = limit.target_value;
+				zone_id = limit.zone;
+				trip_id = limit.trip;
+				if (target_value == TRIP_PT_INVALID_TARGET_STATE) {
+					thd_log_info(
+							"Currently active limit by [%d: %d %d]: ignore\n",
+							zone_id, trip_id, target_value);
+					return THD_SUCCESS;
+				}
 
-			if (cmp_current_state(limit.target_value) < 0) {
-				thd_log_info(
-						"new active zone; next in line  %d trip %d clamp %d\n",
-						limit.zone, limit.trip, limit.target_value);
-				set_curr_state_raw(limit.target_value, zone_id);
-				thd_log_info(
-						"Set : threshold:%d, temperature:%d, cdev:%d(%s), curr_state:%d, max_state:%d\n",
-						set_point, temperature, index, type_str.c_str(),
-						get_curr_state(), max_state);
+			} else if (force) {
+				thd_log_info("forced to min_state \n");
+			} else {
+				// If the deleted entry has a target then on deactivation
+				// set the state to min_state
+				if (_target_value != TRIP_PT_INVALID_TARGET_STATE)
+					target_value = get_min_state();
 			}
-			return THD_SUCCESS;
-
-		} else if (zone_mask != 0 || trip_mask != 0) {
-			thd_log_debug(
-					"skip to reduce current state as this is controlled by two zone or trip actions and one is still on %lx:%lx\n",
-					zone_mask, trip_mask);
-			return THD_SUCCESS;
+		} else {
+			target_value = TRIP_PT_INVALID_TARGET_STATE;
 		}
 	}
+
 	last_action_time = tm;
 
 	curr_state = get_curr_state();
 	if (curr_state == get_min_state()) {
 		control_begin();
 	}
-	if (pid_enable) {
+
+	if (target_value != TRIP_PT_INVALID_TARGET_STATE) {
+		set_curr_state_raw(target_value, zone_id);
+		curr_state = target_value;
+		ret = THD_SUCCESS;
+		thd_log_info("Set : %d, %d, %d, %d, %d\n", set_point, temperature,
+				index, get_curr_state(), max_state);
+
+	} else if (pid_enable) {
 		pid_ctrl.set_target_temp(target_temp);
 		ret = pid_ctrl.pid_output(temperature);
 		ret += get_curr_state();
@@ -271,18 +285,10 @@ int cthd_cdev::thd_cdev_set_state(int set_point, int target_temp,
 
 	return ret;
 }
-;
 
-int cthd_cdev::thd_cdev_set_min_state(int zone_id) {
-	zone_mask &= ~(1 << zone_id);
-	if (zone_mask != 0) {
-		thd_log_debug(
-				"skip to reduce current state as this is controlled by two zone actions and one is still on %x\n",
-				(unsigned int) zone_mask);
-		return THD_SUCCESS;
-	}
+int cthd_cdev::thd_cdev_set_min_state(int zone_id, int trip_id) {
 	trend_increase = false;
-	set_curr_state(min_state, zone_id);
+	thd_cdev_set_state(0, 0, 0, 0, zone_id, trip_id, min_state, true);
 
 	return THD_SUCCESS;
 }
